import prog1javalib.Base;

/*
Windows:
Compile: javac -cp .;prog1javalib.jar Rational.java
Run: java -cp .;prog1javalib.jar Rational

OS X:
Compile: javac -cp .:prog1javalib.jar Rational.java
Run: java -cp .:prog1javalib.jar Rational
*/

public class Rational {
	
	int num;
	int den;
	int gcd;
	
    public Rational(int numerator, int denominator) {
        if (denominator == 0) {
            throw new RuntimeException("zero denominator");
        }
		gcd=greatestCommonDivisor(numerator,denominator);
        num=numerator/gcd;
		den=denominator/gcd;
		if (den<0){
			num=-num;
			den=-den;
		}
    }
    
    public static Rational r(int numerator, int denominator) {
        return new Rational(numerator, denominator);
    }

    @Override
    public String toString() {
		if(num%den==0){
			return ""+num/den;
		}else{
			return num+"/"+den;
		}
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null) return false;
        if (o instanceof Integer) {
            return (num/den==(Integer)o);
        }
        if (o instanceof Rational) {
			Rational r=(Rational)o;
            return (num==r.num)&&(den==r.den);
        }
        return false;
    }

    public Rational plus(Rational r) {
        return r(num*r.den+r.num*den, den*r.den);
    }

    public Rational minus(Rational r) {
        return r(num*r.den-r.num*den, den*r.den);
    }

    public Rational times(Rational r) {
        return r(num*r.num, den*r.den);
    }

    public Rational dividedBy(Rational r) {
        return r(num*r.den, den*r.num);
    }
    
    public Rational inverse() { 
        return r(den, num);
    }

    public Rational plus(int i) {
        return r(num+i*den, den);
    }

    public Rational minus(int i) {
        return r(num-i*den, den);
    }

    public Rational times(int i) {
        return r(num*i, den);
    }

    public Rational dividedBy(int i) {
        return r(num, den*i);
    }
    
    private static int greatestCommonDivisor(int x, int y) {
		x=Math.abs(x);
		y=Math.abs(y);
        if((x*y)==0){
			return x+y;
		}else{
			if(x<y){
				return greatestCommonDivisor(y,x);
			}else{
				return greatestCommonDivisor(x%y,y);	
			}
		}
    }
    
    public static void rationalTest() {
        // 6/8 --> 3/4
        Base.checkExpect(r(6, 8), r(3, 4));
        Base.checkExpect(r(6, 8).toString(), "3/4");

        // 6/-8 --> -3/4
        Base.checkExpect(r(6, -8), r(3, -4));
        Base.checkExpect(r(6, -8).toString(), "-3/4");

        // 1/-2 --> -1/2
        Base.checkExpect(r(1,-2), r(-1, 2));
        Base.checkExpect(r(1,-2).toString(), "-1/2");

        // 1/2 + 3/4 = 5/4
        Base.checkExpect(r(1, 2).plus(r(3, 4)), r(5, 4));
        Base.checkExpect(r(1, 2).plus(r(3, 4)).toString(), "5/4");

        // 1/2 - 3/4 = -1/4
        Base.checkExpect(r(1, 2).minus(r(3, 4)), r(-1, 4));
        Base.checkExpect(r(1, 2).minus(r(3, 4)).toString(), "-1/4");

        // 1/3 + 2/3 = 1
        Base.checkExpect(r(1, 3).plus(r(2, 3)), 1);
        Base.checkExpect(r(1, 3).plus(r(2, 3)).toString(), "1");
                
        // 1/3 - 1/3 = 0
        Base.checkExpect(r(1, 3).minus(r(1, 3)), 0);
        Base.checkExpect(r(1, 3).minus(r(1, 3)).toString(), "0");
                
        // 1/3 - 4/3 = -1
        Base.checkExpect(r(1, 3).minus(r(4, 3)), -1);
        Base.checkExpect(r(1, 3).minus(r(4, 3)).toString(), "-1");
                
        // 1/2 * 4/3 = 2/3
        Base.checkExpect(r(1, 2).times(r(4, 3)), r(2, 3));
                
        // -1/2 * 4/3 = -2/3
        Base.checkExpect(r(-1, 2).times(r(4, 3)), r(-2, 3));
                
        // (1/2) / (4/3) = 3/8
        Base.checkExpect(r(1, 2).dividedBy(r(4, 3)), r(3, 8));
                
        // (-1/2) / (4/3) = -3/8
        Base.checkExpect(r(-1, 2).dividedBy(r(4, 3)), r(-3, 8));
        
        // Integer Test von Rene
        Base.checkExpect(r(1, 3).plus(3), r(10, 3));
        Base.checkExpect(r(1, 3).plus(3).toString(), "10/3");
        
        Base.checkExpect(r(1, 3).minus(3), r(-8, 3));
        Base.checkExpect(r(1, 3).minus(3).toString(), "-8/3");
        
        Base.checkExpect(r(1, 3).times(3), 1);
        Base.checkExpect(r(1, 3).times(3).toString(), "1");
        
        Base.checkExpect(r(1, 3).dividedBy(3), r(1, 9));
        Base.checkExpect(r(1, 3).dividedBy(3).toString(), "1/9");
		
    }

    public static void main(String[] args) {
        rationalTest();
        Base.summary();
    }

}
